#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Script for testing chirpnet lib

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import logging
import unittest
from pathlib import Path

from chirptext import JiCache, WebHelper, FileHelper

# ----------------------------------------------------------------------
# Configuration
# ----------------------------------------------------------------------

TEST_DIR = os.path.dirname(__file__)
TEST_DATA = os.path.join(TEST_DIR, 'data')
TEST_CACHE = os.path.join(TEST_DATA, 'test_cache.db')


def getLogger():
    return logging.getLogger(__name__)


########################################################################

class TestChirpnet(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        if not os.path.isdir(TEST_DATA):
            getLogger().info("Creating TEST DATA folder at {}".format(TEST_DATA))
            FileHelper.create_dir(TEST_DATA)

    def test_chirpnet(self):
        print("Test WebHelper")
        # explicit cache
        web = WebHelper(cache=JiCache(TEST_CACHE))
        u = "https://letuananh.github.io/test/data.json"
        data = web.fetch(u, encoding="utf-8")
        self.assertEqual(data.strip(), '{ "name": "Kungfu Panda" }')
        self.assertIn(u, web.cache)
        # how about just a path as string?
        web2 = WebHelper(TEST_CACHE)
        self.assertIn(u, web2.cache)
        # test path object
        web3 = WebHelper(Path(TEST_CACHE))
        self.assertIn(u, web3.cache)
        # no cache
        web_nocache = WebHelper()
        self.assertIsNone(web_nocache.cache)
        web_nocache = WebHelper(None)
        self.assertIsNone(web_nocache.cache)

    def test_fetch_json(self):
        web = WebHelper(TEST_CACHE)
        u = "https://letuananh.github.io/test/data.json"
        data = web.fetch_json(u, encoding="utf-8")
        # it must be json
        self.assertEqual(data, {"name": "Kungfu Panda"})
        # test fetch non existent URL
        u_error = "https://letuananh.github.io/test/data-null.json"
        print("Fetch nonexistent url ... default quiet=True")
        data = web.fetch_json(u_error, encoding="utf-8")
        self.assertIsNone(data)
        # must raise exception if not quiet
        print("Fetch nonexistent url with quiet=False")
        self.assertRaises(Exception, lambda: web.fetch_json(u_error, encoding="utf-8", quiet=False))


########################################################################

if __name__ == "__main__":
    unittest.main()



import logging


__IGO_AVAILABLE = False
__IGO_TAGGER = None


def __try_import_igo(log=False):
    global __IGO_AVAILABLE
    global __IGO_TAGGER
    try:
        from igo.Tagger import Tagger
        __IGO_TAGGER = Tagger()
        __IGO_AVAILABLE = True
    except Exception as e:
        if log:
            logging.getLogger(__name__).warning('igo-python is not installed. run `pip install igo-python` to install.', exc_info=True)
    return __IGO_AVAILABLE


def igo_available():
    ''' Check if igo-python package is installed '''
    return __IGO_AVAILABLE


__try_import_igo()


def parse(content, *args, **kwargs):
    global __IGO_TAGGER
    tokens = __IGO_TAGGER.parse(content)
    # format: same as mecab
    # 表層形,品詞,品詞細分類1,品詞細分類2,品詞細分類3,活用形,活用型,原形,読み,発音
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Test script for luke

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import unittest
from chirptext.luke import read_swadesh_1971, read_swadesh_ranked, read_swadesh_sign

# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

TEST_DIR = os.path.dirname(os.path.realpath(__file__))
TEST_DATA = os.path.join(TEST_DIR, 'data')


# -------------------------------------------------------------------------------
# Test cases
# -------------------------------------------------------------------------------

class TestLuke(unittest.TestCase):

    def test_swadesh(self):
        s71 = read_swadesh_1971()
        sr = read_swadesh_ranked()
        ss = read_swadesh_sign()
        self.assertEqual(len(s71), 100)
        self.assertEqual(len(sr), 100)
        self.assertEqual(len(ss), 100)
        self.assertEqual(repr(s71[0]), "Word(ID=1, word='I')")
        # are they the same words?
        s71_words = {w.word for w in s71}
        sr_words = {w.word for w in sr}
        self.assertEqual(s71_words, sr_words)
        ss_words = {w.word for w in ss}
        self.assertEqual(len(ss_words), 100)
        # validate words
        same = ss_words.intersection(s71_words)
        diff = ss_words.difference(s71_words)
        self.assertEqual(same, {'die', 'red', 'stone', 'lie', 'leaf', 'grease', 'not', 'tree', 'mountain', 'black', 'kill', 'night', 'egg', 'louse', 'who', 'blood', 'green', 'what', 'rain', 'dry', 'good', 'sit', 'white', 'new', 'man', 'all', 'full', 'fire', 'bird', 'tail', 'woman', 'moon', 'yellow', 'water', 'name', 'feather', 'long', 'sun', 'star', 'stand', 'fish', 'dog', 'person', 'earth'})
        self.assertEqual(diff, {'when', 'wide', 'smooth', 'snow', 'laugh', 'with', 'warm', 'child', 'brother', 'cat', 'sing', 'how', 'correct', 'worm', 'dust', 'sharp', 'pig', 'play', 'meat', 'grass', 'ice', 'dull', 'short', 'animal', 'if', 'river', 'day', 'sea', 'heavy', 'other', 'wife', 'year', 'rope', 'wind', 'narrow', 'because', 'wet', 'salt', 'dirty', 'vomit', 'live', 'father', 'snake', 'husband', 'work', 'old', 'sister', 'mother', 'count', 'flower', 'dance', 'where', 'hunt', 'thin', 'wood', 'bad'})


# -------------------------------------------------------------------------------
# Main
# -------------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

'''
Language profile: UK English

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import codecs
import logging

# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

logger = logging.getLogger(__name__)
MY_DIR = os.path.dirname(__file__)
MY_DATA = os.path.join(MY_DIR, 'data', 'luke')
SWADESH_1971_PATH = os.path.join(MY_DATA, 'swadesh/1971.txt')
SWADESH_RANKED_PATH = os.path.join(MY_DATA, 'swadesh/ranked.txt')
SWADESH_SIGN_PATH = os.path.join(MY_DATA, 'swadesh/sign.txt')


# -------------------------------------------------------------------------------
# Data structures
# -------------------------------------------------------------------------------

class Word(object):
    """ Swadesh word """
    def __init__(self, ID, word, score=0, description='', rank=0):
        self.ID = ID
        self.word = word
        self.score = score
        self.description = description
        self.rank = rank

    def __repr__(self):
        return "Word(ID={}, word={})".format(repr(self.ID), repr(self.word))


def read_swadesh_1971():
    with codecs.open(SWADESH_1971_PATH, 'r', encoding='utf-8') as infile:
        lines = infile.read().splitlines()
        table = [l.split(maxsplit=1) for l in lines if l and not l.startswith("#")]
        words = []
        for idx, row in enumerate(table):
            desc = row[1] if len(row) == 2 else ''
            word = Word(ID=idx + 1, word=row[0], description=desc)
            words.append(word)
        return words


def read_swadesh_ranked():
    with codecs.open(SWADESH_RANKED_PATH, 'r', encoding='utf-8') as infile:
        lines = infile.read().splitlines()
        table = [l.split() for l in lines if l and not l.startswith("#")]
        words = []
        for idx, row in enumerate(table):
            swid, top40, lemma, score = row
            word = Word(ID=swid, word=lemma, score=score, rank=idx + 1)
            words.append(word)
        return words


def read_swadesh_sign():
    with codecs.open(SWADESH_SIGN_PATH, 'r', encoding='utf-8') as infile:
        lines = infile.read().splitlines()
        table = [l for l in lines if l and not l.startswith("#")]
        words = []
        for idx, row in enumerate(table):
            word = Word(ID=idx + 1, word=row.strip())
            words.append(word)
        return words



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Script for testing arsenal lib

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import logging
import unittest

from chirptext.leutile import LOREM_IPSUM
from chirptext import JiCache

# ----------------------------------------------------------------------
# Configuration
# ----------------------------------------------------------------------

TEST_DIR = os.path.dirname(__file__)
TEST_CACHE = os.path.join(TEST_DIR, 'data', 'test_cache.db')


def getLogger():
    return logging.getLogger(__name__)


########################################################################

class TestArsenal(unittest.TestCase):

    def try_cache(self, cache, key, value):
        # delete the blob if it exists
        if key in cache:
            getLogger().debug("Deleting blob with key = {}".format(key))
            cache.delete_blob(key)
        # make sure that it is not there anymore
        self.assertNotIn(key, cache)
        cache.insert_string(key, value)
        # make sure that it is inserted
        self.assertIn(key, cache)
        # make sure that the content is the same
        self.assertEqual(cache.retrieve_string(key), value)

    def test_empty_cache(self):
        TEST_CACHE2 = os.path.join(TEST_DIR, 'test_cache2.db')
        JiCache(TEST_CACHE2)
        self.assertFalse(os.path.exists(TEST_CACHE2))

    def test_arsenal(self):
        print("Test arsenal module")
        cache = JiCache(TEST_CACHE)
        # test insert blob
        self.try_cache(cache, 'lorem', LOREM_IPSUM)
        self.try_cache(cache, 1, 'Key is a number')
        self.try_cache(cache, None, 'None key')


########################################################################

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Test script for anhxa
Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import unittest
import json
import logging
from chirptext.anhxa import IDGenerator, DataObject
from chirptext.anhxa import to_json, to_obj, dumps, update_obj
from chirptext.anhxa import TypedJSONDecoder, TypelessSONEncoder, TypedJSONEncoder


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

TEST_DIR = os.path.dirname(os.path.realpath(__file__))


def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Data Structures
# -------------------------------------------------------------------------------

class PersonifyJSONEncoder(TypedJSONEncoder):

    def __init__(self, *args, **kwargs):
        type_map = {Person: '__person__',
                    Job: '__job__'}
        super(PersonifyJSONEncoder, self).__init__(type_map=type_map, *args, **kwargs)


class PersonifyJSONDecoder(TypedJSONDecoder):

    def __init__(self, *args, **kwargs):
        type_map = {'__person__': Person,
                    '__job__': Job}
        super(PersonifyJSONDecoder, self).__init__(type_map)


class Job(object):

    def __init__(self, job='N/A', sal=0):
        self.job = job
        self.sal = sal

    def __str__(self):
        return ("{} (${}/mth)".format(self.job, self.sal))

    def __eq__(self, other):
        if not other:
            return False
        else:
            return self.job == other.job and self.sal == other.sal


class Person(object):

    def __init__(self, name, age, job=None, pid=None):
        self.name = name
        self.age = age
        self.job = job if job else None
        self.pid = pid

    def change_job(self, job, sal):
        self.job = Job(job, sal)

    def __str__(self):
        return "{} ({}) {}".format(self.name, self.age, self.job if self.job else 'unemployed')

    def __eq__(self, other):
        if not other:
            return False
        return self.name == other.name and self.age == other.age and self.job == other.job


# -------------------------------------------------------------------------------
# Tests
# -------------------------------------------------------------------------------

class TestIDGen(unittest.TestCase):

    def no_even(self, value):
        if value > 0 and value % 2 == 0:
            return True

    def test_idgen(self):
        gen = IDGenerator(id_hook=self.no_even)
        ids = [next(gen) for i in range(5)]
        self.assertEqual(ids, [1, 3, 5, 7, 9])
        # test no hook
        gen2 = IDGenerator()
        ids = [next(gen2) for i in range(5)]
        self.assertEqual(ids, [1, 2, 3, 4, 5])


class TestDataObject(unittest.TestCase):

    def test_dobj(self):
        desc = 'Just an ordinary foo'
        source = {'name': 'foo', 'age': 18, 'desc': desc, 'silent': False}
        obj = DataObject(funny=True)
        obj.hobbies = {'coding', 'languages'}
        obj.update(source, desc='description', silent='muet')
        obj.update(source, 'nonexistence')  # this is OK too
        self.assertEqual(obj.name, 'foo')
        self.assertEqual(obj.age, 18)
        self.assertEqual(obj.description, desc)
        self.assertFalse(obj.muet)
        oj = obj.to_json()
        self.assertNotIn('nonexistence', oj)
        getLogger().debug(oj)


class TestAnhxa(unittest.TestCase):

    def test_new_object(self):
        p = to_obj(Person, {'name': 'Ji', 'age': 10, 'pid': 'p001'})
        p.job = 'dev'
        pj = to_json(p)
        p2 = to_obj(Person, pj)
        self.assertIsInstance(p2, Person)
        self.assertEqual(p, p2)
        #
        pj2 = to_json(pj, 'name', 'age', 'pid')
        p3 = to_obj(Person, pj2)
        self.assertFalse(p3.job)
        # obj to obj
        p4 = to_obj(Person, p3.__dict__)
        self.assertEqual(p3, p4)

    def test_json_encoder(self):
        p = to_obj(Person, {'name': 'Ji', 'age': 10})
        # Person > typed-json
        js = json.dumps(p, cls=PersonifyJSONEncoder)
        # typed-json > Person
        np = json.loads(js, cls=PersonifyJSONDecoder)
        self.assertIsInstance(np, Person)
        npjson = json.dumps(np, cls=PersonifyJSONEncoder)
        self.assertEqual(npjson, js)
        #
        # changing info
        p.change_job('dev', 5000)
        js2 = json.dumps(p, cls=PersonifyJSONEncoder)
        # change salary in JSON
        js2_json = json.loads(js2)
        js2_json['job']['sal'] = 10000
        # convert js2_json to object
        np2 = json.loads(json.dumps(js2_json), cls=PersonifyJSONDecoder)
        self.assertEqual(np2.job.sal, 10000)

    def test_typeless_json(self):
        p = to_obj(Person, {'name': 'Chun', 'age': 15})
        p.change_job('pupil', -500)
        pjstr = json.dumps(p, cls=PersonifyJSONEncoder)
        pjson = json.loads(pjstr)
        self.assertIn('__type__', pjson)
        self.assertEqual(pjson['__type__'], '__person__')
        self.assertIn('__type__', pjson['job'])
        self.assertEqual(pjson['job']['__type__'], '__job__')

        # Test typeless json
        # helper function
        pjtl = dumps(p, indent=2)
        pjson1 = json.loads(pjtl)
        # or full method
        pjtl2 = json.dumps(p, cls=TypelessSONEncoder)
        pjson2 = json.loads(pjtl2)
        self.assertEqual(pjson1, pjson2)

    def test_mapping(self):
        p1 = to_obj(Person, {'name': 'Yin', 'age': 10, 'pid': '001'})
        p2 = to_obj(Person, {'name': 'Yang', 'age': 5, 'pid': '002'})
        p2.job = Job("pupil", -100)
        # p2 > json
        p2json = json.loads(dumps(p2))
        update_obj(p2json, p1, 'name', 'age', 'pid')
        p1.job = to_obj(Job, p2json['job'])
        self.assertEqual(p1, p2)
        # now change p1 salary
        p1.job.sal = -200
        self.assertNotEqual(p1.job.sal, p2.job.sal)

    def test_different_key_mapping(self):
        p1 = to_obj(Person, {'name': 'Yin', 'age': 10, 'pid': '001'})
        o2j_map = {'pid': 'perid'}
        j2o_map = {v: k for k, v in o2j_map.items()}
        p1j = to_json(p1, **o2j_map)
        p2 = to_obj(Person, p1j, **j2o_map)
        self.assertEqual(p1, p2)


# -------------------------------------------------------------------------------
# MAIN
# -------------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

'''
Command-line interface helper

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import logging
import logging.config
import argparse
import json


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

class ChirpCLI(object):
    SETUP_COMPLETED = False


def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Application logic
# -------------------------------------------------------------------------------

def setup_logging(filename, log_dir=None, force_setup=False):
    ''' Try to load logging configuration from a file. Set level to INFO if failed.
    '''
    if not force_setup and ChirpCLI.SETUP_COMPLETED:
        logging.debug("Master logging has been setup. This call will be ignored.")
        return
    if log_dir and not os.path.exists(log_dir):
        os.makedirs(log_dir)
    if os.path.isfile(filename):
        with open(filename) as config_file:
            try:
                config = json.load(config_file)
                logging.config.dictConfig(config)
                logging.info("logging was setup using {}".format(filename))
                ChirpCLI.SETUP_COMPLETED = True
            except Exception as e:
                logging.exception("Could not load logging config")
                # default logging config
                logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.INFO)


def config_logging(args):
    ''' Override root logger's level '''
    if args.quiet:
        logging.getLogger().setLevel(logging.CRITICAL)
    elif args.verbose:
        logging.getLogger().setLevel(logging.DEBUG)


class CLIApp(object):
    ''' A simple template for command-line interface applications '''

    def __init__(self, desc, add_vq=True, add_tasks=True, **kwargs):
        """
        Init an app instance
        """
        self.__parser = argparse.ArgumentParser(description=desc, add_help=False)
        self.__parser.set_defaults(func=None)
        self.__add_vq = add_vq
        self.__config_logging = config_logging
        if 'config_logging' in kwargs:
            self.__config_logging = kwargs['config_logging']
        if add_vq:
            self.add_vq(self.__parser)
        self.__show_version_func = None
        if 'show_version' in kwargs:
            self.add_version_func(kwargs['show_version'])
        if add_tasks:
            task_desc = kwargs['task_desc'] if 'task_desc' in kwargs else 'Task to be done'
            self.__tasks = self.__parser.add_subparsers(help=task_desc)
        else:
            self.__tasks = None
        self.__logger = None
        self.__name = kwargs['logger'] if 'logger' in kwargs else __name__

    def add_task(self, task, func=None, **kwargs):
        ''' Add a task parser '''
        if not self.__tasks:
            raise Exception("Tasks subparsers is disabled")
        if 'help' not in kwargs:
            if func.__doc__:
                kwargs['help'] = func.__doc__
        task_parser = self.__tasks.add_parser(task, **kwargs)
        if self.__add_vq:
            self.add_vq(task_parser)
        if func is not None:
            task_parser.set_defaults(func=func)
        return task_parser

    def add_vq(self, parser):
        ''' Add verbose & quiet options '''
        group = parser.add_mutually_exclusive_group()
        group.add_argument("-v", "--verbose", action="store_true")
        group.add_argument("-q", "--quiet", action="store_true")

    def add_version_func(self, show_version):
        ''' Enable --version and -V to show version information '''
        if callable(show_version):
            self.__show_version_func = show_version
        else:
            self.__show_version_func = lambda cli, args: print(show_version)
        self.parser.add_argument("-V", "--version", action="store_true")

    @property
    def logger(self):
        ''' Lazy logger '''
        if self.__logger is None:
            self.__logger = logging.getLogger(self.__name)
        return self.__logger

    def run(self, func=None):
        ''' Run the app '''
        args = self.parser.parse_args()
        if self.__add_vq is not None and self.__config_logging:
            self.__config_logging(args)
        if self.__show_version_func and args.version and callable(self.__show_version_func):
            self.__show_version_func(self, args)
        elif args.func is not None:
            args.func(self, args)
        elif func is not None:
            func(self, args)
        else:
            self.parser.print_help()

    @property
    def parser(self):
        return self.__parser

    @property
    def tasks(self):
        return self.__tasks



# -*- coding: utf-8 -*-

'''
Chirp Net - Web utilities

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
from io import BytesIO
import gzip
import logging
import json
from urllib.request import Request, urlopen
from urllib.error import URLError
from urllib.parse import urlparse, urlunparse, parse_qs, urlencode, quote

from chirptext.arsenal import JiCache


# ----------------------------------------------------------------------
# Configuration
# ----------------------------------------------------------------------

def getLogger():
    return logging.getLogger(__name__)


# ----------------------------------------------------------------------
# Functions
# ----------------------------------------------------------------------

class SmartURL(object):
    ''' Smart URL supports URL manipulation '''

    def __init__(self, raw_url, quoted=False):
        self.raw = raw_url
        self.p = None
        self.scheme = None
        self.netloc = None
        self.opath = None
        self.path = None
        self.params = None
        self.query = None
        self.fragment = None
        if self.raw:
            self.process(quoted)

    def get_filename(self):
        return os.path.basename(self.p.path)

    def get_filename_noext(self):
        return os.path.splitext(self.get_filename())[0]

    def get_file_ext(self):
        return os.path.splitext(self.get_filename())[1]

    def process(self, quoted=False):
        ''' Parse an URL '''
        self.p = urlparse(self.raw)
        self.scheme = self.p.scheme
        self.netloc = self.p.netloc
        self.opath = self.p.path if not quoted else quote(self.p.path)
        self.path = [x for x in self.opath.split('/') if x]
        self.params = self.p.params
        self.query = parse_qs(self.p.query, keep_blank_values=True)
        self.fragment = self.p.fragment

    def __str__(self):
        flatd = [(k, v[0] if len(v) == 1 else v) for k, v in self.query.items()]
        return urlunparse((self.scheme, self.netloc, '/'.join(self.path), self.params, urlencode(flatd), self.fragment))


class WebHelper(object):
    ''' a wget like utility for Python '''

    def __init__(self, cache=None):
        if cache is None or isinstance(cache, JiCache):
            self.cache = cache
        else:
            self.cache = JiCache(location=str(cache))

    @staticmethod
    def encode_url(url):
        return str(SmartURL(url, quoted=True))

    def fetch(self, url, encoding=None, force_refetch=False, nocache=False, quiet=True):
        ''' Fetch a HTML file as binary'''
        try:
            if not force_refetch and self.cache is not None and url in self.cache:
                # try to look for content in cache
                logging.debug('Retrieving content from cache for {}'.format(url))
                return self.cache.retrieve_blob(url, encoding)
            encoded_url = WebHelper.encode_url(url)
            req = Request(encoded_url, headers={'User-Agent': 'Mozilla/5.0'})
            # support gzip
            req.add_header('Accept-encoding', 'gzip, deflate')
            # Open URL
            getLogger().info("Fetching: {url} |".format(url=url))
            response = urlopen(req)
            content = response.read()
            # unzip if required
            if 'Content-Encoding' in response.info() and response.info().get('Content-Encoding') == 'gzip':
                # unzip
                with gzip.open(BytesIO(content)) as gzfile:
                    content = gzfile.read()
            # update cache if required
            if self.cache is not None and not nocache:
                if url not in self.cache:
                    self.cache.insert_blob(url, content)
            return content.decode(encoding) if content and encoding else content
        except URLError as e:
            error_message = "Unknown fetching error"
            if hasattr(e, 'reason'):
                error_message = 'We failed to reach {}. Reason: {}'.format(url, e.reason)
            elif hasattr(e, 'code'):
                error_message = 'The server couldn\'t fulfill the request. Error code: {}'.format(e.code)
            # done!
            if not quiet:
                raise
            else:
                getLogger().debug(error_message)
        return None

    def fetch_json(self, *args, **kwargs):
        output = self.fetch(*args, **kwargs)
        if output is not None:
            return json.loads(output)
        else:
            return None

    def download(self, url, path, force_refetch=False, nocache=False):
        ''' Download a file at $url and save it to $path
        '''
        # Enable cache
        if os.path.isfile(path):
            getLogger().info("File exists, download task skipped -> {path}".format(path=path))
            return True
        try:
            # Open URL
            getLogger().info("Downloading: {url} -> {path}".format(url=url, path=path))
            response = self.fetch(url, force_refetch=force_refetch, nocache=nocache)
            if response is not None:
                # Download file
                local_file = open(path, "wb")
                local_file.write(response)
                local_file.close()
                # Finished
                return True
            else:
                return False
        except Exception as e:
            if hasattr(e, 'reason'):
                getLogger().exception('We failed to reach a server. Reason: %s' % (e.reason,))
            elif hasattr(e, 'code'):
                getLogger().exception("The server couldn't fulfill the request. Error code: {code}".format(code=e.code))
            else:
                # everything is fine
                getLogger().exception("Unknown error: %s" % (e,))
        return False



# -*- coding: utf-8 -*-

'''
Chirp Text - Minimalist Text Processing Library

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

########################################################################

from .__version__ import __author__, __email__, __copyright__, __maintainer__
from .__version__ import __credits__, __license__, __description__, __url__
from .__version__ import __version_major__, __version_long__, __version__, __status__

from .leutile import uniquify, header, confirm, piter
from .leutile import Timer, Counter, FileHub, TextReport, FileHelper, AppConfig
from .arsenal import JiCache
from .anhxa import DataObject
from .chirpnet import SmartURL, WebHelper
from . import texttaglib as ttl

__all__ = ["uniquify", "header", "confirm", "piter",
           "Timer", "Counter",
           "TextReport", "FileHub", "FileHelper", "AppConfig",
           "JiCache", "SmartURL", "WebHelper",
           "DataObject", "ttl",
           "__version__", "__author__", "__description__", "__copyright__"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Test script for deko module

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import unittest
import logging

from chirptext import TextReport
from chirptext import deko
from chirptext.deko import KATAKANA, simple_kata2hira
from chirptext.deko import wakati, tokenize, tokenize_sent, analyse, parse
from chirptext.deko import MeCabSent, DekoText
# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

TEST_DIR = os.path.dirname(os.path.realpath(__file__))
txt = '雨が降る。'
txt2 = '猫が好きです。\n犬も好きです。'
txt3 = '猫が好きです。\n犬も好きです。\n鳥は'
txt4 = '猫が好きです。犬も好きです。鳥は'


def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Test cases
# -------------------------------------------------------------------------------

class TestTool(unittest.TestCase):

    ALL_MAPPING = '''ぁァ|あア|ぃィ|いイ|ぅゥ|うウ|ぇェ|えエ|ぉォ|おオ|かカ|がガ|きキ|ぎギ|くク|ぐグ|けケ|げゲ|こコ|ごゴ|さサ|ざザ|しシ|じジ|すス|ずズ|せセ|ぜゼ|そソ|ぞゾ|たタ|だダ|ちチ|ぢヂ|っッ|つツ|づヅ|てテ|でデ|とト|どド|なナ|にニ|ぬヌ|ねネ|のノ|はハ|ばバ|ぱパ|ひヒ|びビ|ぴピ|ふフ|ぶブ|ぷプ|へヘ|べベ|ぺペ|ほホ|ぼボ|ぽポ|まマ|みミ|むム|めメ|もモ|ゃャ|やヤ|ゅュ|ゆユ|ょョ|よヨ|らラ|りリ|るル|れレ|ろロ|ゎヮ|わワ|ゐヰ|ゑヱ|をヲ|んン|ゔヴ|ゕヵ|ゖヶ|'''

    def test_kata2hira(self):
        rp = TextReport.string()
        for k in KATAKANA[1:87]:
            h = simple_kata2hira(k)
            rp.write(h, k, '|', separator='')
        expected = TestTool.ALL_MAPPING
        self.assertEqual(rp.content(), expected)

    def test_check_kana(self):
        self.assertTrue(deko.is_kana(''))
        self.assertRaises(ValueError, lambda: deko.is_kana(None))
        self.assertTrue(deko.is_kana('ひらがな'))
        self.assertTrue(deko.is_kana('カタカナ'))
        self.assertTrue(deko.is_kana(TestTool.ALL_MAPPING.replace('|', '')))
        # false
        self.assertFalse(deko.is_kana(TestTool.ALL_MAPPING))  # with pipe
        self.assertFalse(deko.is_kana('巡り会う'))  # with kanji
        self.assertFalse(deko.is_kana('すき です'))  # with a space


class TestDeko(unittest.TestCase):

    def test_mecab(self):
        print("Testing mecab")
        tokens = parse(txt)
        self.assertTrue(tokens[-1].is_eos)

    def test_mecab_bin_loc(self):
        mbin_original = deko.get_mecab_bin()
        if mbin_original:
            self.assertTrue(mbin_original == 'mecab' or mbin_original.endswith('mecab.exe'))
        with self.assertLogs('chirptext.dekomecab', level='WARNING') as log:
            mecab_custom_loc = 'C:\\mecab\\mecab-console.exe'
            deko.set_mecab_bin(mecab_custom_loc)
            self.assertEqual(deko.get_mecab_bin(), mecab_custom_loc)
            if not os.path.isfile(mecab_custom_loc):
                self.assertEqual(log.output, ['WARNING:chirptext.dekomecab:Provided mecab binary location does not exist C:\\mecab\\mecab-console.exe'])
            # set it back
        with self.assertLogs('chirptext.dekomecab', level='WARNING') as log:
            deko.set_mecab_bin(mbin_original)
            if not os.path.isfile(mbin_original):
                self.assertEqual(log.output, ['WARNING:chirptext.dekomecab:Provided mecab binary location does not exist ' + mbin_original])

    def test_dekomecab(self):
        # try parsing text using mecab binary
        self.assertRaises(FileNotFoundError, lambda: parse(txt, mecab_loc='/usr/bin/path/to/mecab-binary-app'))
        self.assertRaises(FileNotFoundError, lambda: analyse(txt, mecab_loc='/usr/bin/path/to/mecab-binary-app'))
        self.assertRaises(FileNotFoundError, lambda: DekoText.parse(txt, mecab_loc='/usr/bin/path/to/mecab-binary-app'))
        self.assertRaises(FileNotFoundError, lambda: MeCabSent.parse(txt, mecab_loc='/usr/bin/path/to/mecab-binary-app'))

    def test_mecab_lines(self):
        out = parse(txt2)
        self.assertGreaterEqual(len(out), 11)

    def test_wakati(self):
        tks = wakati(txt).splitlines()[0]
        self.assertEqual(tks, '雨 が 降る 。 ')

    def test_deko(self):
        tokenized = tokenize(txt)
        self.assertEqual(tokenized, ['雨', 'が', '降る', '。'])

    def test_tokenize_sents(self):
        tokens = parse(txt2)
        sents = tokenize_sent(tokens, txt2)
        texts = [x.text for x in sents]
        self.assertEqual(texts, ['猫が好きです。', '犬も好きです。'])

    def test_analyse2(self):
        sents = DekoText.parse(txt)
        self.assertEqual(len(sents), 1)
        self.assertEqual(len(sents[0]), 5)
        self.assertEqual(str(sents[0]), '雨 が 降る 。')
        # 2 sentences
        sents = DekoText.parse(txt2)
        self.assertEqual(len(sents), 2)
        self.assertEqual(len(sents[0]), 6)
        self.assertEqual(len(sents[1]), 6)
        self.assertEqual(sents[0].words, ['猫', 'が', '好き', 'です', '。'])
        self.assertEqual(str(sents[1]), '犬 も 好き です 。')
        # 2 sentences
        getLogger().debug("Tokenized: ", parse(txt2))
        sents = DekoText.parse(txt2, splitlines=False)
        getLogger().debug("last test: {} - {} sents".format(sents, len(sents)))
        self.assertEqual(len(sents), 2)
        self.assertEqual(len(sents[0]), 5)
        self.assertEqual(len(sents[1]), 5)
        self.assertEqual(str(sents[0]), '猫 が 好き です 。')
        self.assertEqual(str(sents[1]), '犬 も 好き です 。')

    def test_analyse(self):
        sents = analyse(txt2, format='txt')
        self.assertEqual(sents, '猫 が 好き です 。\n犬 も 好き です 。')
        # test sent tokenizing using MeCab
        tokens = parse(txt2)
        sents = tokenize_sent(tokens)
        # using analyse function
        sents = analyse(txt2, splitlines=False, format='txt').split('\n')
        self.assertEqual(sents, ['猫 が 好き です 。', '犬 も 好き です 。'])
        # to html
        h = analyse(txt, format='html')
        self.assertEqual(h, '<ruby><rb>雨</rb><rt>あめ</rt></ruby> が <ruby><rb>降る</rb><rt>ふる</rt></ruby>。 ')
        # to csv
        c = analyse(txt, format='csv')
        e = '''雨	名詞	一般	*	*	*	*	雨	アメ	アメ
が	助詞	格助詞	一般	*	*	*	が	ガ	ガ
降る	動詞	自立	*	*	五段・ラ行	基本形	降る	フル	フル
。	記号	句点	*	*	*	*	。	。	。
EOS									
'''
        self.assertEqual(c, e)

    def test_pos(self):
        sent = parse(txt)
        self.assertTrue(sent[-1].is_eos)
        poses = [tk.pos3() for tk in sent if not tk.is_eos]
        self.assertEqual(poses, ['名詞-一般', '助詞-格助詞-一般', '動詞-自立', '記号-句点'])
        for tk in sent:
            getLogger().debug(tk.pos3())

    def test_deko_ttl(self):
        sent = parse(txt).to_ttl()
        getLogger().debug("Sent: {}".format(sent))
        for tk in sent:
            getLogger().debug("{} - {}".format(tk, tk.pos))
        sj = sent.to_json()
        getLogger().debug(sj)
        expected = {'text': '雨が降る。', 'tokens': [{'cfrom': 0, 'cto': 1, 'text': '雨', 'lemma': '雨', 'pos': '名詞-一般', 'tags': [{'label': 'あめ', 'type': 'reading', 'source': 'mecab'}]}, {'cfrom': 1, 'cto': 2, 'text': 'が', 'lemma': 'が', 'pos': '助詞-格助詞-一般', 'tags': [{'label': 'が', 'type': 'reading', 'source': 'mecab'}]}, {'cfrom': 2, 'cto': 4, 'text': '降る', 'lemma': '降る', 'pos': '動詞-自立', 'tags': [{'label': 'ふる', 'type': 'reading', 'source': 'mecab'}]}, {'cfrom': 4, 'cto': 5, 'text': '。', 'lemma': '。', 'pos': '記号-句点', 'tags': [{'label': '。', 'type': 'reading', 'source': 'mecab'}]}]}
        self.assertEqual(sj, expected)
        # test doc to ttl
        doc = DekoText.parse(txt3, splitlines=True)
        ttl_doc = doc.to_ttl()
        self.assertEqual(len(ttl_doc), 3)
        for ds, ts in zip(doc, ttl_doc):
            self.assertEqual(ds.text, ts.text)
            for dtk, ttk in zip(ds, ts):
                self.assertEqual(dtk.surface, ttk.text)
                self.assertEqual(dtk.pos3(), ttk.pos)

    def test_analyse_multiple_sents(self):
        msent = parse(txt4)
        getLogger().debug(msent.tokens)
        sent = msent.to_ttl()
        expected_tokens = ['猫', 'が', '好き', 'です', '。', '犬', 'も', '好き', 'です', '。', '鳥', 'は']
        tokens = [tk.text for tk in sent.tokens]
        self.assertEqual(tokens, expected_tokens)
        # check reading
        readings = [tk.get_tag('reading').label for tk in sent.tokens]
        expected_readings = ['ねこ', 'が', 'すき', 'です', '。', 'いぬ', 'も', 'すき', 'です', '。', 'とり', 'は']
        self.assertEqual(readings, expected_readings)
        # try tokenizing sentences
        sents = tokenize_sent(msent.tokens, txt4)
        expected = [['猫が好きです。', ['猫', 'が', '好き', 'です', '。']], ['犬も好きです。', ['犬', 'も', '好き', 'です', '。']], ['鳥は', ['鳥', 'は']]]
        actual = [[sent.surface, sent.words] for sent in sents]
        self.assertEqual(actual, expected)

    def test_not_split(self):
        doc = DekoText.parse(txt3, splitlines=False)
        docx = DekoText.parse(txt3, splitlines=True)
        doc_words = [x.surface for x in doc]
        docx_words = [x.surface for x in docx]
        getLogger().debug("splitlines: {} | surface={}".format(docx, docx_words))
        getLogger().debug("not splitlines: {} | surface={}".format(doc, doc_words))
        self.assertEqual(doc_words, docx_words)
        getLogger().debug(doc)

    def test_func_alias(self):
        sent = deko.parse(txt)
        self.assertEqual(sent.words, ['雨', 'が', '降る', '。'])
        doc = deko.parse_doc(txt3, splitlines=False)
        self.assertEqual(len(doc), 3)


# -------------------------------------------------------------------------------
# Main
# -------------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main()



import os
import logging
import platform
import subprocess

# Try to use mecab-python3 if it's available
MECAB_PYTHON3 = False
MECAB_LOC = 'mecab'  # location of mecab's binary package


def __try_import_mecab(log=False):
    global MECAB_PYTHON3
    global MECAB_LOC
    MECAB_PYTHON3 = False
    try:
        import MeCab
        MeCab.Tagger().parse("Pythonが好きです。")
        MECAB_PYTHON3 = True
    except:
        # use flex-mecab
        try:
            if platform.system() == 'Windows':
                if os.path.isfile("C:\\Program Files (x86)\\MeCab\\bin\\mecab.exe"):
                    MECAB_LOC = "C:\\Program Files (x86)\\MeCab\\bin\\mecab.exe"
                elif os.path.isfile("C:\\Program Files\\MeCab\\bin\\mecab.exe"):
                    MECAB_LOC = "C:\\Program Files\\MeCab\\bin\\mecab.exe"
                else:
                    MECAB_LOC = "mecab.exe"
            elif os.path.isfile('/usr/local/bin/mecab'):
                MECAB_LOC = '/usr/local/bin/mecab'
            else:
                MECAB_LOC = "mecab"
        except:
            pass
        if log:
            logging.getLogger(__name__).warning("mecab-python3 could not be loaded. mecab binary package will be used ({})".format(MECAB_LOC))
    return MECAB_PYTHON3


__try_import_mecab()


def _register_mecab_loc(location):
    ''' Set MeCab binary location '''
    global MECAB_LOC
    if not os.path.isfile(location):
        logging.getLogger(__name__).warning("Provided mecab binary location does not exist {}".format(location))
    logging.getLogger(__name__).info("Mecab binary is switched to: {}".format(location))
    MECAB_LOC = location


def _get_mecab_loc():
    ''' Get MeCab binary location '''
    return MECAB_LOC


def run_mecab_process(content, *args, **kwargs):
    ''' Use subprocess to run mecab '''
    encoding = 'utf-8' if 'encoding' not in kwargs else kwargs['encoding']
    mecab_loc = kwargs['mecab_loc'] if 'mecab_loc' in kwargs else None
    if mecab_loc is None:
        mecab_loc = MECAB_LOC
    proc_args = [mecab_loc]
    if args:
        proc_args.extend(args)
    output = subprocess.run(proc_args,
                            input=content.encode(encoding),
                            stdout=subprocess.PIPE)
    output_string = os.linesep.join(output.stdout.decode(encoding).splitlines())
    return output_string


def parse(content, *args, **kwargs):
    ''' Use mecab-python3 by default to parse JP text. Fall back to mecab binary app if needed '''
    global MECAB_PYTHON3
    if 'mecab_loc' not in kwargs and MECAB_PYTHON3 and 'MeCab' in globals():
        return MeCab.Tagger(*args).parse(content)
    else:
        return run_mecab_process(content, *args, **kwargs)


def wakati(content):
    return parse(content, '-Owakati')



# -*- coding: utf-8 -*-

# chirptext's package version information
__author__ = "Le Tuan Anh"
__email__ = "tuananh.ke@gmail.com"
__copyright__ = "Copyright (c) 2012, Le Tuan Anh"
__credits__ = []
__license__ = "MIT License"
__description__ = "ChirpText is a collection of text processing tools for Python."
__url__ = "https://letuananh.github.io/chirptext/"
__maintainer__ = "Le Tuan Anh"
__version_major__ = "0.1"
__version__ = "{}a19".format(__version_major__)
__version_long__ = "{} - Alpha".format(__version_major__)
__status__ = "Prototype"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Test Dao Phay library

Latest version can be found at https://github.com/letuananh/chirptext

:copyright: (c) 2012 Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import unittest
import logging
from chirptext import daophay


########################################################################

def getLogger():
    return logging.getLogger(__name__)


class TestDaoPhay(unittest.TestCase):

    def vn_sorted_arrays(self, inputs, expected):
        actual = daophay.sorted(inputs)
        self.assertEqual(expected, actual)

    def test_vn_sorted(self):
        print("Test vn sorting")
        self.vn_sorted_arrays([], [])
        self.vn_sorted_arrays(['anh', 'an'], ['an', 'anh'])
        self.vn_sorted_arrays(['anh', 'an', 'an,'], ['an', 'an,', 'anh'])
        self.vn_sorted_arrays(['uống', 'uốn', 'ương'], ['uốn', 'uống', 'ương'])
        self.vn_sorted_arrays(['sữa めいじ', 'sửa めいじ', 'sứ めいじ'], ['sửa めいじ', 'sữa めいじ', 'sứ めいじ'])
        # test inplace sorting
        a_list = ['mạ', 'má', 'mã', 'ma', 'mà', 'mả']
        a_list.sort(key=daophay.vnorder)
        getLogger().debug(a_list)
        expected = ['ma', 'mà', 'mả', 'mã', 'má', 'mạ']
        self.assertEqual(a_list, expected)
        # with other characters
        a_list = ['mà み', 'ma み', 'mà ま', 'ma ま']
        a_list.sort(key=daophay.vnorder)
        getLogger().debug(a_list)
        expected = ['ma ま', 'ma み', 'mà ま', 'mà み']
        self.assertEqual(a_list, expected)


########################################################################

if __name__ == "__main__":
        unittest.main()
